/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "popd.h"
#include "helpers.h"

#ifdef USE_SHADOW
#include <shadow.h>
#endif

#ifdef USE_PWENCRYPT
extern char *pw_encrypt(const char *, const char *);
# define crypt pw_encrypt
#endif

static void clear_authinfo(struct passwd *pwe)
{
  safe_free((void **) &pwe->pw_name);

  /* clear the password record before freeing it */
  
  if(pwe->pw_passwd)
  {
    memset(pwe->pw_passwd, 0, strlen(pwe->pw_passwd));
    safe_free((void **) &pwe->pw_passwd);
  }
  
  safe_free((void **) &pwe->pw_gecos);
  safe_free((void **) &pwe->pw_dir);
  safe_free((void **) &pwe->pw_shell);
}
  
int do_user(struct context *ctx, const char *arg)
{
  char *s;
  
#ifdef USE_SHADOW
  struct spwd *spe;
  struct passwd *pwe;
#endif
  
  if((s = strchr(arg, '/')))
  {
    *s++ = '\0';
    safe_free((void **) &ctx->what);
    ctx->what = safe_strdup(s);
  }
  else
    safe_free((void **)&ctx->what);
  
  if((pwe = getpwnam(arg)))
  {
    if(!ctx->authinfo)
    {
      ctx->authinfo = safe_malloc(sizeof (struct passwd));
      memset(ctx->authinfo, 0, sizeof(struct passwd));
    }

    clear_authinfo(ctx->authinfo);
    
    ctx->authinfo->pw_name = safe_strdup(pwe->pw_name);
    ctx->authinfo->pw_passwd = safe_strdup(pwe->pw_passwd);
    ctx->authinfo->pw_uid = pwe->pw_uid;
    ctx->authinfo->pw_gid = pwe->pw_gid;
    ctx->authinfo->pw_gecos = safe_strdup(pwe->pw_gecos);
    ctx->authinfo->pw_dir = safe_strdup(pwe->pw_dir);
    ctx->authinfo->pw_shell = safe_strdup(pwe->pw_shell);

#ifdef USE_SHADOW
    if(ctx->authinfo && (spe = getspnam(arg)))
    {
      safe_free((void **) &ctx->authinfo->pw_passwd);
      ctx->authinfo->pw_passwd = safe_strdup(spe->sp_pwdp);
      endspent();
    }
#endif
  }
  else if(ctx->authinfo)
  {
    clear_authinfo(ctx->authinfo);
    safe_free((void **) &ctx->authinfo);
  }
    
  pop_ok("Please give us your password.");
  
  return 0;
}

#define BADAUTH_THRESHOLD 3

int do_pass(struct context*ctx, const char *arg)
{
  char *pw;
  
  if(ctx->badauth >= BADAUTH_THRESHOLD || !ctx->authinfo)
  {
    pop_err("Bad password.");
    return -1;
  }
  
  if(!ctx->authinfo->pw_passwd)
    pw = "xx";
  else
    pw = ctx->authinfo->pw_passwd;
  
  if(!strcmp(crypt(arg, pw), pw))
  {
    if(setgid(ctx->authinfo->pw_gid) == -1 || 
       setuid(ctx->authinfo->pw_uid) == -1)
      pop_die("Can't setgid/setuid.", NULL);

    open_folder(ctx);

    pop_ok("Authentication successful.");
    return 0;
  }

  /* else */
  
  ctx->badauth++;
  pop_err("Bad password.");
  pop_log("Bad password.");
  
  return -1;
}

