/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include "helpers.h"
#include "popd.h"

static char *get_folder_from_cfg(struct context *);
static void read_msgs (struct context *, const char *, int);

void open_folder(struct context *ctx)
{
  char *folder;
  
  folder = get_folder_from_cfg(ctx);
  
  if(chdir(folder) == -1)
    pop_die(NULL, "Can't chdir!");
  
  safe_free((void **) &folder);
  
  read_msgs(ctx, "new/", M_NEW);
  read_msgs(ctx, "cur/", M_SEEN);
  
}

int do_update(struct context *ctx)
{
  size_t i;
  char *scratch;
  char *tgt;
  
  for(i = 0; i < ctx->n_msgs; i++)
  {
    if(ctx->mailbox[i].flags & M_DELETE)
    {
      scratch = safe_malloc(strlen(ctx->mailbox[i].fname) + 10);
      sprintf(scratch, "%s/%s", ctx->mailbox[i].flags & M_NEW ?
	      "new" : "cur", ctx->mailbox[i].fname);
      
      unlink(scratch);
      safe_free((void **) &scratch);
    }
    else if((ctx->mailbox[i].flags & (M_SEEN | M_NEW)) == (M_SEEN | M_NEW))
    {
      scratch = safe_malloc(strlen(ctx->mailbox[i].fname) + 10);
      tgt     = safe_malloc(strlen(ctx->mailbox[i].fname) + 10);
      
      sprintf(scratch, "%s/%s", "new", ctx->mailbox[i].fname);
      sprintf(tgt,     "%s/%s:2,S", "cur", ctx->mailbox[i].fname);
      
      /* XXX - check the return value */
      
      rename(scratch, tgt);
      
      safe_free((void **) &scratch);
      safe_free((void **) &tgt);
    }
  }
  return 0;
}

static void read_msgs (struct context *ctx, const char *dir, int flags)
{
  DIR *dptr;
  struct dirent *de;
  struct stat sb;
  char *scratch;
  
  if((dptr = opendir(dir)) == NULL)
    pop_die(NULL, "Can't opendir().");
  
  while((de = readdir(dptr)))
  {

    if(*de->d_name == '.')
      continue;
    
    if(ctx->n_msgs == ctx->p_msgs)
      safe_realloc((void **) &ctx->mailbox,
		   sizeof(struct msg) * (ctx->p_msgs += 10));
    
    scratch = safe_malloc(strlen(dir) + strlen(de->d_name) + 2);
    sprintf(scratch, "%s/%s", dir, de->d_name);
    
    if(stat(scratch, &sb) == -1)
      pop_die(NULL, "Can't stat.");
    
    safe_free((void **) &scratch);
    
    ctx->mailbox[ctx->n_msgs].fname = safe_strdup(de->d_name);
    ctx->mailbox[ctx->n_msgs].flags = flags;
    ctx->mailbox[ctx->n_msgs].oct = sb.st_size;
    ctx->s_mbx += sb.st_size;
    
    ctx->n_msgs++;
  }
  
  closedir(dptr);
}

static char *get_folder_from_cfg(struct context *ctx)
{
  const char *tag;
  FILE *fp;
  char *fname;
  char *dir;
  char *t;
  char buff[BUFLEN];
  int have_tag;
  
  if(ctx->what)
    tag = ctx->what;
  else
    tag = "default";

  fname = safe_malloc(strlen(ctx->authinfo->pw_dir) + 10);
  sprintf(fname, "%s/.poprc", ctx->authinfo->pw_dir);
  
  if(!(fp = fopen(fname, "r")))
    pop_die("No configuration file.", "No configuration file.");
  
  safe_free((void **) &fname);

  have_tag = 0;
  dir = NULL;
  
  while(!dir && fgets(buff, sizeof(buff), fp))
  {
    if(!(t = strchr(buff, '\n')))
      pop_die(NULL, "Long line");

    if(!(t = strtok(buff, "= \n")) || strcmp(tag, t))
      continue;
    
    if(!(t = strtok(NULL, "= \n")))
      continue;
    
    dir = t;
  }
    
  fclose(fp);
  
  if(!dir)
    pop_die(NULL, "Unkown mailbox.");
  return safe_strdup(dir);
}
