/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <syslog.h>

#include "helpers.h"
#include "popd.h"

char *safe_strdup(const char *s)
{
  char *p;
  
  if(!(p = strdup(s)))
    pop_die("Out of memory", NULL);
  
  return p;
}

void *safe_malloc(size_t s)
{
  void *p;
  
  if(!(p = malloc(s)))
    pop_die("Out of memory", NULL);
  
  return p;
}

void *safe_realloc(void **p, size_t s)
{
  void *q;
  
  if(!p) return NULL;
  
  if(!*p)
    return((*p = safe_malloc(s)));
  else
  {  
    if(!(q = realloc(*p, s)))
      pop_die("Out of memory", NULL);
    
    *p = q;
    return *p;
  }
}
  

void safe_free(void **p)
{
  if(!p || !*p)
    return;
  
  free(*p);
  *p = NULL;
}

static int in_die = 0;

void pop_die(const char *log, const char *net)
{
  
  if(in_die)
    exit(1);
  
  in_die = 1;
  
  if(net)
    pop_err(net);
  
  if(log)
    pop_log(log);
  
  exit(1);
}

void pop_log(const char *log)
{
  syslog(LOG_WARNING, "%s", log);
}
    
