/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "helpers.h"
#include "popd.h"


void pop_err (const char *s)
{
  char *scratch;
  
  scratch = safe_malloc(strlen(s) + 7);
  sprintf(scratch, "-ERR %s", s);
  pop_wrln(scratch);
  safe_free((void **)&scratch);
}

void pop_ok (const char *s)
{
  char *scratch;
  
  scratch = safe_malloc(strlen(s) + 7);
  sprintf(scratch, "+OK %s", s);
  pop_wrln(scratch);
  safe_free((void **) &scratch);
}

void pop_wrln (const char *s)
{
  char *t;
  
  if((t = strchr(s, '\n')))
    *t = '\0';
     
  printf("%s\r\n", s);
  fflush(stdout);
}

void pop_wrmln (const char *s)
{
  if(*s == '.')
    putc('.', stdout);
  
  pop_wrln(s);
}

char *pop_getln (void)
{
  char *buff = NULL;
  size_t l = 0;
  size_t u = 0;
  int c;
  
  while(1)
  {
    if((c = fgetc(stdin)) == '\n')
      break;

    if(c == EOF)
    {
      wannaquit = 1;
      break;
    }
    
    if(c == '\r')
    {
      int d = fgetc(stdin);
      
      if(d == '\n')
	break;
      else
	ungetc(d, stdin);

    }
    
    if(u == l)
    {
      l += BUFLEN;
      safe_realloc((void **) &buff, l);
    }
    
    buff[u++] = (char) c;
  }
  
  if(buff)
    buff[u] = '\0';
  
  return buff;
}
