/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef _POPD_H
#define _POPD_H

#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>

#define M_DELETE (1 << 0)
#define M_SEEN  (1 << 1)
#define M_NEW   (1 << 2)

#define S_AUTHUSER  	(1 << 0)
#define S_AUTHPASS	(1 << 1)
#define S_TRANS 	(1 << 2)
#define S_QUIT  	(1 << 3)

#define S_AUTH (S_AUTHUSER | S_AUTHPASS)

#define CF_ARG (1 << 0)
#define CF_MAYBE (1 << 1)
#define CF_NOLOGARG (1 << 2)

#define BUFLEN 4096

struct msg
{
  char *fname;
  size_t oct;
  int flags;
};


struct context
{
  int state;
  struct msg *mailbox;
  size_t p_msgs;
  size_t n_msgs;
  size_t s_mbx;
  int badauth;
  struct passwd *authinfo;
  char *what;
};

void pop_err (const char *);
void pop_ok  (const char *);
void pop_wrln (const char *);
void pop_wrmln (const char *);
char *pop_getln (void);


int do_user(struct context *, const char *);
int do_pass(struct context *, const char *);
int do_stat(struct context *);
int do_list(struct context *, const char *);
int do_retr(struct context *, const char *);
int do_dele(struct context *, const char *);
int do_rset(struct context *);
int do_update(struct context *);

void open_folder(struct context *ctx);

int wannaquit;

#endif
