/*
 *  popd - a lean and hopefully secure pop3 implementation for
 *  maildir folders.
 * 
 *  Copyright (C) 1998  Thomas Roessler <roessler@guug.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#include "popd.h"
#include "helpers.h"

int do_stat(struct context *ctx)
{
  char scratch[BUFLEN];
  
  snprintf(scratch, sizeof(scratch),
	   "%d %d",
	   ctx->n_msgs,
	   ctx->s_mbx);
  
  pop_ok(scratch);
  
  return 0;
}


int do_list(struct context *ctx, const char *arg)
{
  size_t i;
  char scratch[BUFLEN];
  
  if(arg)
  {
    i = strtoul(arg, NULL, 10);
    
    if(i == 0 || i > ctx->n_msgs || ctx->mailbox[i-1].flags & M_DELETE)
    {
      pop_err("No such message.");
      return -1;
    }
    
    snprintf(scratch, sizeof(scratch),
	     "%d %d", i, ctx->mailbox[i - 1].oct);
    
    pop_ok(scratch);
  }
  else
  {
    pop_ok("Message list follows.");
    
    for(i = 0; i < ctx->n_msgs; i++)
    {
      if(!(ctx->mailbox[i].flags & M_DELETE))
      {
	snprintf(scratch, sizeof(scratch),
		 "%d %d", i + 1, ctx->mailbox[i].oct);
	pop_wrmln(scratch);
      }
    }
    
    pop_wrln(".");
  }
  
  return 0;
}



int do_retr(struct context *ctx, const char *arg)
{
  size_t idx;
  FILE *fp;
  char c;
  int is_first;
  char *fn;
  char scratch[BUFLEN];
  
  idx = strtoul(arg, NULL, 10);
  
  if(idx == 0 || idx > ctx->n_msgs ||
     ctx->mailbox[idx - 1].flags & M_DELETE)
  {
    pop_err("No such message.");
    return -1;
  }
  
  idx--;
  
  fn = safe_malloc(strlen(ctx->mailbox[idx].fname) + 10);
  sprintf(fn, "%s/%s", ctx->mailbox[idx].flags & M_NEW ? "new" : "cur",
	   ctx->mailbox[idx].fname);
  
  if(!(fp = fopen(fn, "r")))
  {
    pop_err("Can't open file.");
    safe_free((void **) &fn);
    return -1;
  }
  
  safe_free((void **) &fn);
  
  snprintf(scratch, sizeof(scratch), "%d", ctx->mailbox[idx].oct);
  pop_ok(scratch);
  
  is_first = 1;
  
  while((c = fgetc(fp)) != EOF)
  {
    if(is_first && c == '.')
      putc('.', stdout);
    
    if(c == '\n')
    {
      putc('\r', stdout);
      is_first = 1;
    } 
    else
      is_first = 0;
    
    putc(c, stdout);
  }
  
  fclose(fp);
  
  if(!is_first)
    pop_wrln("");
  
  pop_wrln(".");

  ctx->mailbox[idx].flags |= M_SEEN;
  
  return 0;
}

int do_dele(struct context *ctx, const char *arg)
{
  size_t idx;
  
  idx = strtoul(arg, NULL, 10);
  
  if(idx == 0 || idx > ctx->n_msgs)
  {
    pop_err("No such message.");
    return -1;
  }
  
  ctx->mailbox[idx-1].flags |= M_DELETE;
  pop_ok("");
  return 0;
}

int do_rset(struct context *ctx)
{
  size_t i;
  
  for(i = 0; i < ctx->n_msgs; i++)
    ctx->mailbox[i].flags &= ~M_DELETE;
  
  pop_ok("");
  return 0;
}

